<?php
namespace Ric;
defined( 'ABSPATH' ) || exit;



/**
* Main plugin class
*
* @since 1.0.0
* @param none
* @return none;
*/
class RicMain{
    
    private static $instance = null;
 
    /**
    * Get instance
    *
    * @since 1.0.0
    * @param none
    * @return none;
    */
    public static function get_instance() {
        if ( null == self::$instance ) {
                self::$instance = new self;
        }
        return self::$instance;
    } 
    
    
    /**
    * __construct
    *
    * @since 1.0.0
    * @param none
    * @return none;
    */
    private function __construct() {

     
      

        
        /*
         * functions
         */
        require RIC_DIR.'inc/functions.php';
      


        /*
        * wp hooks
        */
        require RIC_DIR.'inc/hooks.php';
        
          
        /*
         * admin fields
         */
        require RIC_DIR.'inc/admin_fields.php';
        
        /*
         * ajax
         */
        require RIC_DIR.'inc/public_ajax.php';
         
        
        /* save def settings */
        $saved= \get_option(RIC_SETTINGS_KEY);
        if(!$saved){
        
            $data=[];
            $sections=Fields::getFieldsSettings();
            foreach ($sections as $section=>$fields){
                 foreach ($fields as $field){
                     $key=$field['key'];
                     $def=$field['def'];
                     if(!isset($data[$section])){
                         $data[$section]=[];
                     }
                         $data[$section][$key] = $def;
                 }
            }
            \update_option(RIC_SETTINGS_KEY, $data);


 

    
        }
        
    
        if(is_admin()){

            require RIC_DIR.'inc/admin_menu.php';
            require RIC_DIR.'inc/admin_ajax.php';
            require RIC_DIR.'inc/status.php';
   
            
            /*
             * main setting link in plugins list admin
             */
            \add_filter( 'plugin_action_links_' . \plugin_basename(RIC_FILE ), array( $this, 'plugin_action_links' ) );
            
            /*
            * load admin
            */
            \add_action( 'admin_init', array( $this, 'admin_init' ), 12, 1 );



            
            /*
            * admin menu
            */
        //    add_action('admin_menu', 'ric_plugin_admin_main_menu_cb');
            
            /*
             * hook sync post after save 
             */
            \add_action('save_post', array( $this, 'admin_save_post_cb' ) );

        }else if(Func::isActive()){
 
          
            /*
             * public staff
             */
 
            require RIC_DIR.'inc/public.php';
              
            /*
             * front end scrips
             */
            \add_action( 'wp_enqueue_scripts',  array( $this, 'register_front_plugin_scripts' ) );
            
            /*
             * register cron job five minutes
             */
            if (! \wp_next_scheduled ( 'ric_everyfiveminutes_cron_jobs' )) {
                \wp_schedule_event(time()+1, 'everyfiveminutes', 'ric_everyfiveminutes_cron_jobs');
            }
            /*
             * register cron job hourly
             */
            if (! \wp_next_scheduled ( 'ric_hourly_cron_jobs' )) {
                \wp_schedule_event(time()+1, 'hourly', 'ric_hourly_cron_jobs');
            }
        }
        
        /*
         * run cron job
         */
        \add_action('ric_everyfiveminutes_cron_jobs',array( $this, 'do_cron_jobs_everyfiveminutes' ) );
        \add_action('ric_hourly_cron_jobs',array( $this, 'do_cron_jobs_hourly' ) );
        


        /*
         * hook plugion loaded
         */
        \do_action('ric_plugin_loaded');




        \add_filter( 'label_row_after_pro',function($html){
            if(strlen($html)){
                $html.=' <a href="https://www.myricerca.com/" target="blank">'.esc_html__("Here",'ric').'</a>';
            }
            return $html;
        },5);


         

    }
    

    /**
    * Run cron job
    *
    * @since 1.0.0
    * @param none
    * @return none;
    */
    public function do_cron_jobs_everyfiveminutes(){
        /*
         * sync posts by cron job
         */
        Func::doSyncCronPosts();

     
    }
    /**
    * Run cron job do_cron_jobs_hourly
    *
    * @since 1.0.0
    * @param none
    * @return none;
    */
    public function do_cron_jobs_hourly(){

        
 

      
        //clear deleted & not relative items

        // unmatches the settings
        $reg_post_types = Func::getPostTypes();
        $selectes = Func::getPostTypesToSync();

     
        if(!empty($reg_post_types)){
            $unmateched = [];
            foreach($reg_post_types as $reg_post_type){
                if(!in_array($reg_post_type,$selectes)){
                    $unmateched[]=$reg_post_type;
                }
            }

       
           
            if(!empty($unmateched)){
                try{


                    Db::deleteByPostType('items',$unmateched);

                   
                }catch(Exception $e){

                }
            }
        }


        //deleted unselected terms
        $selectes = Func::getTaxonomoesToSync();
        $taxonomies = Func::getTaxonomies(); 
 
        if(!empty($taxonomies)){
            $unmateched = [];
            foreach($taxonomies as $tax){
                if(!in_array($tax,$selectes)){
                    $unmateched[]=$tax;
                }
            }
            if(!empty($unmateched)){
                try{

                    Db::deleteByPostType('taxonomies',$unmateched);
 
                }catch(Exception $e){
                }
            }
        }



        
    }
    
    
    /**
    * Run admin init callbacks
    *
    * @since 1.0.0
    * @param none
    * @return none;
    */
    public function admin_init() {
        /*
         * register admin css & js
         */
        \add_action('admin_enqueue_scripts', array( $this, 'admin_plugin_styles_cb' ));
        
        /*
         * put some ric variables in admin footer
         */
        \add_action( 'admin_footer', array( $this, 'admin_plugin_footer_cb' ));
    }
    
  
    /**
    * Print admin footer js variables
    *
    * @since 1.0.0
    * @param none
    * @return none;
    */
    public function admin_plugin_footer_cb(){
        $args=[];
        $args['assets_url'] = Func::getAssets();
        $args['crons'] = Func::getCronsList();
        $args['strings'] = [];
        $args['strings']['msg1'] =esc_html__('Done','ric');
        $args['strings']['msg2'] =esc_html__('Finalizing.','ric');

        ?><script>
            var ricAdminConfig = <?php echo \json_encode($args); ?>;
        </script><?php
    }
    
  
    /**
    * print admin css & js
    *
    * @since 1.0.0
    * @param none
    * @return none;
    */
    public function admin_plugin_styles_cb(){
        /*
         * main admin css
         */
        \wp_register_style( RIC_SLUG.'-admin', Func::getAssets('css/admin.css'),array(),RIC_VER);
        \wp_enqueue_style( RIC_SLUG.'-admin');
        //add rtl support
      //  \wp_style_add_data(  RIC_SLUG.'-admin', 'rtl', 'replace' );
     
        /*
        * main admin js
        */ 
	    \wp_enqueue_script(RIC_SLUG.'-js',Func::getAssets('js/admin.js'),[],RIC_VER,true);
    }
    

    /**
    * print front css & js
    *
    * @since 1.0.0
    * @param none
    * @return none;
    */
    public function register_front_plugin_scripts(){
      //  $suffix  = defined('SCRIPT_DEBUG') && SCRIPT_DEBUG ? '' : '.min';
        $suffix  = '';
        \wp_enqueue_script(RIC_SLUG.'-front-js',Func::getAssets('js/front'.$suffix.'.js') ,array('jquery'),RIC_VER,true);
        \wp_register_style( RIC_SLUG.'-front-css', Func::getAssets('css/front'.$suffix.'.css'),array(),RIC_VER);
        \wp_enqueue_style( RIC_SLUG.'-front-css');
        \wp_style_add_data(  RIC_SLUG.'-front-css', 'rtl', 'replace' );
    }


    /**
    * append main settings menu 
    *
    * @since 1.0.0
    * @param none
    * @return none;
    */
    public function plugin_action_links( $actions ) {
        $custom_actions = array();
        $custom_actions['settings'] =sprintf( '<a href="%s" >%s</a>', admin_url('admin.php?page=ric_plugin_admin_menu_page'), esc_html__( 'Settings', 'ric' ) ); 

        // add the links to the front of the actions list
        return array_merge( $custom_actions, $actions );
    }

    /**
    * sync post after save post
    *
    * @since 1.0.0
    * @param none
    * @return none;
    */
    public function admin_save_post_cb(){
        global $_POST;
        $types=Func::getPostTypesToSync();


        
        $pid = \absint($_POST['post_ID']);
        $ptype = \wp_kses_post($_POST['post_type']);

        if(!empty($types) && !empty($_POST) && in_array($ptype, $types)){
            $savedData= \get_post(intval($pid));
            Func::syncPost($savedData);
        }
    }
}